#include "vorbisDecoder.h"

#ifdef HAVE_LIBVORBIS

#include <iostream>
#include "oggBOSExtractorFactory.h"

/* We need this information only internally to interpret the
 * vorbis packet header */
/*
struct VorbisPackHeader {
  char dataType:1;
  char block:1;
  char blsht:6;
};
*/

VorbisDecoder::VorbisDecoder(int8 _streamID)
 : MediaOutputDecoder(_streamID), initCount(0)
{

}

VorbisDecoder::~VorbisDecoder()
{

}

std::string VorbisDecoder::getInfoString()
{
  return("");

/*  std::stringstream stream;

  stream << "Vorbis Stream";

  return(stream.str());
*/
}

void VorbisDecoder::init()
{
  /* initialize the info and comment handler structs */
  vorbis_info_init(&vorbisInfo);
  vorbis_comment_init(&vorbisComment);

  /* set the state machine */
  setInitialized();

}

MediaOutputDecoder& VorbisDecoder::operator<<(OggPacket packet)
{
  /* if the stream is not initialized, initialize the first structs */
  if (!isInitialized()) {
    OggBOSExtractorFactory::extractInformation(packet, streamInformation);
    granulePos = OggBOSExtractorFactory::extractPositionInterpreter(streamInformation);
    init();
  }

  /* if the stream is not configured the first three packets are header
   * packets */
  if (!isConfigured()) {
    if (initCount < streamInformation.numOfHeaderPackets) {

      if (vorbis_synthesis_headerin(&vorbisInfo, &vorbisComment, packet.obj()) != 0)
        std::cerr << "VorbisDecoder::operator<<: packet is not a header\n";
      else {
        /* store the header packets for later use */
        packet.setStreamHeader();
        packet.setStreamType(ogg_vorbis);
        headerList.push_back(packet);
        initCount++;
      }

      /* if all headers have been decoded, initialize the decoder itself (this is called configuration) */
      if (initCount == streamInformation.numOfHeaderPackets) {

        vorbis_synthesis_init(&vorbisDspState,&vorbisInfo);
        vorbis_block_init(&vorbisDspState,&vorbisBlock);

        setConfigured();
      }

    }
    return(*this);
  }

  /* while inserting data into the stream, we do not
   * decode. We just store the packets and will decode them
   * on demand */

  packet.setStreamType(ogg_vorbis);
  packetList.push_back(packet);

  /* has there not been a packet in the queue before */
  if (isEmpty()) {
    /* set the internal state */
    setAvailable();
  }

  /* count the audio packets, to have a glimps of the actual position */
  packetCount++;
  return(*this);

}

MediaOutputDecoder& VorbisDecoder::operator>>(OggPacket& packet)
{

  /* is there at lease one packet available? */
  if (isEmpty()) {
    std::cerr << "TheoraDecoder::operator>>: no packet available\n";
    return(*this);
  }

  /* return the first packet in the list */
  packet = packetList.front();
  packetList.pop_front();

  /* when there is no packet left, set state machine to empty */
  if (packetList.empty()) {
    if (packet.isEOS())
      setEndOfStream();
    else
      setEmpty();
  }

  /* set the actual granule position of this packet
   * (actually we do not care about the position from the page info */
  granulePos->setStreamPosition(packet);

  packet.setStreamType(ogg_vorbis);
  packet.setStreamNo(streamID);

  return(*this);
}

VorbisDecoder& VorbisDecoder::operator>>(AudioPacket audioPacket)
{
  return(*this);
}

void VorbisDecoder::getHeaderList(std::vector<OggPacket>& list)
{
  list = headerList;
}

vorbis_comment& VorbisDecoder::getComment()
{
  return(vorbisComment);
}

uint32 VorbisDecoder::getPositionOfNextPacket()
{
  return(packetCount - packetList.size());
}

int64 VorbisDecoder::getGranulePosOfNextPacket()
{
  return(granulePos->getPosition());
}

double VorbisDecoder::getTimeOfNextPacket()
{
  return(granulePos->getActTime());
}

#endif // WITH_LIBVORBIS
